package net.thegreshams.gosuZfsUtility.impl

uses java.io.File
uses java.util.Arrays
uses java.util.logging.*
uses java.lang.RuntimeException

uses net.thegreshams.gosuZfsUtility.*

class GosuZfsEmulator implements GosuZfsShell {

	final var _headers : List<String> = {}
	final var _snapshots : List<String> = {}
	final var _zfsVolume : String
	final var _delimeter : String
	final var _volumeSnapshotSeparator : String
	
	final static var LOGGER = Logger.getLogger( GosuZfsEmulator.Type.Name )
	final static var LOGGER_ENTRY = "ENTRY"
	final static var LOGGER_EXIT = "EXIT"
	final static var LOGGER_EXECUTING = "EXECUTING: "
	
	var SUDO = "sudo"
	var COMMAND_LIST_ALL = "zfs list -t all"
	var COMMAND_DESTROY = "zfs destroy"
	var COMMAND_CREATE = "zfs snapshot"
	var RESULT_SNAPSHOT_CREATED = "Snapshot created."
	var RESULT_SNAPSHOT_ALREADY_EXISTS = "Snapshot already exists."
	var RESULT_DESTROYED_PREFIX = "Destroyed "
	var RESULT_DESTROYED_SUFFIX = " snapshots."
	
	var _use_keyword_sudo : boolean = true

	

	construct( relativePathToTestFileContainingSampleListOfSnapshots : String, 
			   useKeywordSudo : boolean,
			   zfsVolume : String,
			   delimeter : String,
			   volumeSnapshotSeparator : String) 
   {
		LOGGER.info( LOGGER_ENTRY )
	    
	    // if the supplied-arguments appear to be valid, continue
	    if( relativePathToTestFileContainingSampleListOfSnapshots.trim().HasContent 
	    	and zfsVolume.trim().HasContent
	    	and delimeter.HasContent
	    	and volumeSnapshotSeparator.trim().HasContent ) 
    	{
	    	
	    	this._use_keyword_sudo = useKeywordSudo
	    	this._zfsVolume = zfsVolume.trim()
	    	this._delimeter = delimeter
	    	this._volumeSnapshotSeparator = volumeSnapshotSeparator.trim()
	    	
	    	initializeCommandStrings()
	    	
	    	LOGGER.info( "Initiating for volume '${this._zfsVolume}' using delimeter '${this._delimeter}' and separator '${this._volumeSnapshotSeparator}'." )
	    	
	    	parseFileContentsAndStoreData( relativePathToTestFileContainingSampleListOfSnapshots.trim(), this._delimeter )
	    	LOGGER.info( "Parsed test-file into sample-list of snapshots -- found ${this._snapshots.Count} snapshots." )
	    	
	    } else {
	        
	        LOGGER.severe( "Called without supplying valid construct-arguments." )
	    }
	    
	    LOGGER.info( LOGGER_EXIT )
	}

	override function execute( shellCommand : String) : String {
	    
	    var result = ""	    
	    LOGGER.info( LOGGER_ENTRY )

	    if( shellCommand.trim().HasContent ) {
	        
	        shellCommand = shellCommand.trim()
	    
	    	LOGGER.info( LOGGER_EXECUTING + shellCommand )
		    
		    if( shellCommand == COMMAND_LIST_ALL ) {
		        
		        result = command_getListAll()
		        
		    } else if( shellCommand.startsWith( COMMAND_DESTROY ) ) {
		        
		        result = command_destroy( shellCommand.split( COMMAND_DESTROY )[1] )
		        
		    } else if( shellCommand.startsWith( COMMAND_CREATE ) ) {
		        
		        result = command_create( shellCommand.split( COMMAND_CREATE )[1] )
		    }
		    
	    }
	    
	    LOGGER.info( LOGGER_EXIT )	  
	    return "${result}\n"
	}
	
	private function command_create( snapshotToCreate : String ) : String {
	    
	    var result = ""
	    LOGGER.info( LOGGER_ENTRY )
	    
	    snapshotToCreate = snapshotToCreate.trim()
	    	    
	    var snapshotAlreadyExists = false
	    for( snapshotLine in this._snapshots ) {
	        var snapshot = snapshotLine.split( " " )[0]
	        if( snapshot == snapshotToCreate ) {
	            snapshotAlreadyExists = true
	            break
	        }
	    }
	    
	    if( snapshotAlreadyExists ) {
	        LOGGER.info( "Snapshot already exists: ${snapshotToCreate}" )
	        result = RESULT_SNAPSHOT_ALREADY_EXISTS
	    } else {
	        LOGGER.info( "Adding snapshot: ${snapshotToCreate}" )
	        this._snapshots.add( snapshotToCreate )
	        result = RESULT_SNAPSHOT_CREATED
	    }
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function TEMP_test_TEMP_destroy ( snapshotToDestroy : String ) {
	    
	    var targetedSnapshots = this._snapshots.where( \ l -> l.split( " " ).first() == snapshotToDestroy )
	    targetedSnapshots.each( \ l -> LOGGER.info( "Found ${l}" ) )
	    
	}
	
	private function command_destroy( snapshotToDestroy : String ) : String {
	    
	    var result = ""
	    LOGGER.info( LOGGER_ENTRY )
	    
	    snapshotToDestroy = snapshotToDestroy.trim()
	    
	    // iterate over each of the snapshots, targeting each one that matches the one indicated for destruction
	    var targetedSnapshots = this._snapshots.where( \ snapshot -> snapshot.split( " " ).first() == snapshotToDestroy )
	    
	    // log each of the snapshots that match the target
	    targetedSnapshots.each( \ snapshot -> LOGGER.info( "Found matching snapshot to destroy: ${snapshot}" ) )
	    
	    // destroy all of the target-matches
	    LOGGER.info( "Removing ${targetedSnapshots.Count} snapshots." )
	    this._snapshots.removeAll( targetedSnapshots )
	    result = RESULT_DESTROYED_PREFIX + targetedSnapshots.Count + RESULT_DESTROYED_SUFFIX
	    
	    LOGGER.info( LOGGER_EXIT )
	    return result
	}
	
	private function command_getListAll() : String {
	    
		var result = ""
		LOGGER.info( LOGGER_ENTRY )
		
		// merge the headers and snapshots into a single array, then stuff them into a string with each element separated by the delimeter
	    result = {this._headers, this._snapshots}.flatten().join( this._delimeter )
		
		// if the merged-result doesn't contain anything, replace it with an empty-string (to make sure we don't get a string containing 'null')
		result = result.HasContent ? result : ""
		
		LOGGER.info( LOGGER_EXIT )	
	    return result
	}
	
	private function parseFileContentsAndStoreData( pathToFile : String, delimeter : String ) {

	    var file = new File( pathToFile )
	    var contents = file.read()
	    
	    var doneWithHeader = false
	    var targetLine = this._zfsVolume + this._volumeSnapshotSeparator
	    for( line in contents.split( this._delimeter ) ) {

	        if( line.trim().contains( targetLine ) ) {	
	                     
	            this._snapshots.add( line )
	            doneWithHeader = true
	            
	        } else if( not doneWithHeader ) {
	            
	            this._headers.add( line )
	            
	        }
	        
	    }
	    
	} 
	
	private function initializeCommandStrings() {
	    
	    var sudoPrefix = ""
	    if( this._use_keyword_sudo ) {
	        sudoPrefix = this.SUDO + " "
	    }
	    
	    this.COMMAND_CREATE = sudoPrefix + this.COMMAND_CREATE
	    this.COMMAND_DESTROY = sudoPrefix + this.COMMAND_DESTROY
	    this.COMMAND_LIST_ALL = sudoPrefix + this.COMMAND_LIST_ALL
	}
	
}
